//Title: OutputStream Class
//Platform: BeOS 5
//Version: 2002.03.19
//Description:
//	A class for supplying a stream of media (BBuffers) to a BBufferProducer's
//	output, from a BMediaTrack of a BMediaFile.
	
//Copyright (C) 2001, 2002 Kevin H. Patterson
//Available for use under the terms of the BSD license agreement:
//See the associated file "License.txt" for details.


#ifndef _OutputStream_h
	#define _OutputStream_h

	#include <MediaFile.h>
	#include <MediaTrack.h>
	#include <MediaNode.h>
	#include <Buffer.h>
	#include <BufferGroup.h>
	#include <Locker.h>

	#include "Log.h"
	#include <iostream.h>

	class MediaPlayerNode;
	
	class OutputStream {
	private:
		status_t iInitCheck;
		MediaPlayerNode * iOwner;
		BMediaFile * iMediaFile;
		BMediaTrack * iMediaTrack;
		BLocker * iTrackLocker;
		BBufferGroup * iBufferGroup;
		BBufferGroup * iPrivateBufferGroup;
		media_output iOutput;
		media_format iPreferredFormat;
		media_format iDecodedFormat;
		volatile thread_id iProducerThreadID;
		volatile bool iRunning;
		volatile bool iConnected;
		volatile bool iOutputEnabled;
		volatile bigtime_t iMediaTime;
		volatile bigtime_t iMediaStartTime;
		volatile bigtime_t iProcessingLatency;
		volatile bigtime_t iDownstreamLatency;
		volatile bigtime_t iExtraLatency;
		
	public:
		OutputStream( MediaPlayerNode * uOwner, BMediaFile * uMediaFile, const int32 uTrackIndex );
		~OutputStream();
		status_t InitCheck() const;

		status_t SpecializeFormat( media_format * uMediaFormat );
		const media_format * PreferredFormat() const;
		bool IsAudio() const;
		bool IsVideo() const;

		void Preroll();

		void Connect(
			status_t error,
			const media_destination & destination,
			const media_format & format,
			char * outName );
		void Disconnect( const media_destination & where );
		bool Connected() const;

		media_output * Output();
		void EnableOutput( const bool enabled );
		bool OutputEnabled() const;

		status_t SetBufferGroup( BBufferGroup * uBufferGroup );
		BBufferGroup * BufferGroup() const;

		void SetDownstreamLatency( bigtime_t uDownstreamLatency );
		void SetExtraLatency( bigtime_t uExtraLatency );
		bigtime_t GetLatency();

		void Start( bigtime_t performance_time );
		void Stop();
		void Stop( bigtime_t performance_time );
		void TimeWarp( bigtime_t performance_time );
		void Seek( bigtime_t media_time, bigtime_t performance_time );

	private:
		status_t CreatePrivateBufferGroup();
		status_t DeletePrivateBufferGroup();

		static int32 RunProducerThread( void * data );
		int32 ProducerThread();
		
	public:
		bigtime_t Duration() const;
		bigtime_t Position() const;
	};
	
#endif
